#include "Communicator.h"
#include "LibInterface.h"
#include <chrono>
#include <iostream>
#include <thread>

struct Main {
  vitejte::Communicator communicator;

  void onPatientsChanged() {
    std::cout << "Patients changed:\n";
    for (const auto &patient : communicator.getPatients()) {
      std::cout << patient << "\n";
    }
  }

  void onError(const std::string &error) {
    std::cout << "Error: " << error << "\n";
  }

  void onStateChange(vitejte::VitejteState state, double until) {
    std::cout << "State: " << static_cast<int>(state) << " until: " << until
              << "\n";
  }
};

__stdcall void callbackPatientChange(void *ptr) {
  auto &main = *reinterpret_cast<Main *>(ptr);
  main.onPatientsChanged();
}
__stdcall void callbackError(void *ptr, const char *msg) {
  auto &main = *reinterpret_cast<Main *>(ptr);
  main.onError(msg);
}
__stdcall void callbackStateChange(void *ptr, int state, double until) {
  auto &main = *reinterpret_cast<Main *>(ptr);
  main.onStateChange(static_cast<vitejte::VitejteState>(state), until);
}

void setup(Main &main) {
  auto &commRef = main.communicator;

  const auto devices = commRef.findDevices();
  std::cout << "Choose device: \n";
  for (auto i = 0; i < devices.size(); ++i) {
    std::cout << "[" << i << "]\t" << devices[i] << "\n";
  }

  std::string userInput;
  std::cout << "Choose device by index: ";
  std::cin >> userInput;
  const unsigned int index = std::atoi(userInput.c_str());
  std::cout << "Chosen ID: " << devices[index] << "\n";
  commRef.setDevice(devices[index]);
  std::cout << "Set device\n";
  std::cout << "Registering\n";
  commRef.registerClient(44444);

  commRef.setPatientChangeCallback(callbackPatientChange, &main);
  commRef.setErrorCallback(callbackError, &main);
  commRef.setStatChangeCallback(callbackStateChange, &main);
}

int main() {
  using namespace std::chrono_literals;
  Main main;

  setup(main);

  std::string input;
  std::cout << "Type 'exit' to quit, c<id> to clear a patient\n";
  while (input != "exit") {
    std::cin >> input;
    if (input.empty()) {
      continue;
    }
    if (input[0] == 'c') {
      const auto idStr = input.substr(1);
      if (!idStr.empty()) {
        const auto id = std::atoi(idStr.c_str());
        main.communicator.clearPatient(id);
      }
    }
  }
  main.communicator.unregisterClient();
  std::cout << "quitting..." << std::endl;
  std::this_thread::sleep_for(5s);
  vitejte::freeLibrary();

  return 0;
}
