unit VitejteInterface;

interface

uses
  Classes, Windows, Controls, Sysutils, Dialogs,
  WinUtils, Defs;

type
  TPacient = packed record
  public
    id: Integer;
    registrace: TDateTime;
    odbaveni: TDateTime;
    titulPred: PAnsiChar;
    jmeno: PAnsiChar;
    prijmeni: PAnsiChar;
    titulZa: PAnsiChar;
    rc: PAnsiChar;
    zp: PAnsiChar;
    duvod: PAnsiChar;
    frontaId: Integer;
    stav: Integer;
    jazyk: PAnsiChar;
    karta: Integer;
  end;

  PPacient = ^TPacient;

  TPacientArray = array [0 .. Pred(1000000) div SizeOf(PPacient)] of PPacient;
  PPacientArray = ^TPacientArray;

  TCommHandle = Pointer;
  TCallbackHandle = Pointer;
  PStringArray = ^TStringArray;
  TStringArray = array [0 .. Pred(1000000) div SizeOf(PAnsiChar)] of PAnsiChar;

  TStdcallCallback = procedure(ACustomPayload: Pointer); stdcall;
  // AState nabyva hodnot [0, 1, 2, 3, 4, 5]
  // 0 - neznamy stav
  // 1 - Bezna funkce
  // 2 - Mimo ordinacni dobu
  // 3 - Rezim nastaveni
  // 4 - Prestavka
  // 5 - Zprava pacientum
  TStdcallCallbackState = procedure(ACustomPayload: Pointer; aState: Integer;
    aUntil: TTime); stdcall;
  // AMessage je uvolneno automaticky po dokonceni vykonavani callbacku.
  TStdcallCallbackError = procedure(ACustomPayload: Pointer;
    AMessage: PAnsiChar); stdcall;

  /// Chyby vracene z funkci:
  /// 0 - OK
  /// 1 - neplatny handle objekt

  /// <summary>
  /// Alokuje objekt pro komunikaci se zarizenim.
  /// </summary>
  /// <returns>
  /// Pointer na alokovany objekt.
  /// </returns>
function createCommunicationHandle: TCommHandle; stdcall;
/// <summary>
/// Alokuje objekt pro ulozeni callback funkci.
/// </summary>
/// <returns>
/// Pointer na alokovany objekt
/// </returns>
function createCallbackHandle: TCallbackHandle; stdcall;
/// <summary>
/// Vyhledani zarizeni na vsech dostupnych rozhrani pomoci UDP broadcast.
/// Blokujici operace.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <param name="AIDList">
/// Vystupni parametr, ktery obsahuje identifikatory zarizeni.
/// NUTNO DEALOKOVAT POMOCI freeStringArray()
/// </param>
/// <param name="ASize">
/// Vystupni parametr obsahujici pocet identifikatoru
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function findDevices(AHandle: TCommHandle; var AIDList: PStringArray;
  var Size: Integer): Integer; stdcall;
/// <summary>
/// Nastavi zarizeni pro komunikaci.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <param name="AID">
/// ID zarizeni pro komunikaci, musi byt jedno ze seznamu vraceneho pomoci findDevices(...)
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function setDevice(AHandle: TCommHandle; AID: PAnsiChar): Integer; stdcall;
/// <summary>
/// Vrati pacienty, kteri byli zaregistrovani do zarizeni.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <param name="APatients">
/// Vystupni parametr, ktery obsahuje ukazatele na informace o pacientovi.
/// NUTNO DEALOKOVAT POMOCI freePatients()
/// </param>
/// <param name="ASize">
/// Vystupni parametr obsahujici pocet pacientu
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function getPat(AHandle: TCommHandle; var APatients: PPacientArray;
  var ACount: Integer): Integer; stdcall;
/// <summary>
/// Zapne nove vlakno, ktere udrzuje periodickou komunikaci s Vitejte.
/// Zajistuje aktualnost dat v pripade vypadku paketu.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function startPingTimer(AHandle: TCommHandle): Integer; stdcall;
/// <summary>
/// Zastavi vlakno aktivovane pomoci startPingTimer(...) a ukonci periodickou komunikaci s Vitejte.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function stopPingTimer(AHandle: TCommHandle): Integer; stdcall;
/// <summary>
/// Registruje lokalni server pro TCP komunikaci se zarizenim.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <param name="APort">
/// Port pro lokalni TCP server, slouzijici k prijmu udalosti ze zarizeni (novy pacient atp.)
/// </param>
/// <param name="AQueues">
/// Seznam front, ze kterych chce prijemce cist pacienty. Pole kladnych cisel reprezentovanych jako string.
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function registerClient(AHandle: TCommHandle; APort: Integer;
  AQueues: PAnsiChar): Integer; stdcall;
/// <summary>
/// Odregistrovani klienta od Vitejte.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function unRegisterClient(AHandle: TCommHandle): Integer; stdcall;
/// <summary>
/// Ziskani IP adresy aktualne pripojeneho Vitejte.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <param name="AIP">
/// Vystupni parametr obsahujici IP adresu Vitejte.
/// NUTNO UVOLNIT POMOCI freeString(...)
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function getVitejteIP(AHandle: TCommHandle; var AIP: PAnsiChar)
  : Integer; stdcall;
/// <summary>
/// Ziskani IDaktualne pripojeneho Vitejte.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <param name="AIP">
/// Vystupni parametr obsahujici IP adresu Vitejte.
/// NUTNO UVOLNIT POMOCI freeString(...)
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function getVitejteID(AHandle: TCommHandle; var AIP: PAnsiChar)
  : Integer; stdcall;
/// <summary>
/// Ziskani informaci o aktualne pripojenem Vitejte.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <param name="AIP">
/// Vystupni parametr obsahujici IP adresu Vitejte.
/// NUTNO UVOLNIT POMOCI freeString(...)
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function getInfo(AHandle: TCommHandle; var AIP: PAnsiChar): Integer; stdcall;
/// <summary>
/// Nastavi vybranemu pacientovi priznak odbaven.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <param name="APatientID">
/// ID ciloveho pacienta.
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function clearPatient(Handle: TCommHandle; APatientID: Integer)
  : Integer; stdcall;
/// <summary>
/// Registrace callbacku pro zmenu ve fronte pacientu.
/// </summary>
/// <param name="AHandle">
/// Pointer na callback objekt ziskany pomoci createCallbackHandle()
/// </param>
/// <param name="AOnChanged">
/// Callback
/// </param>
/// <param name="ACustomPayload">
/// Parametr, ktery bude predan callbacku pri jeho vyvolani.
/// </param>
/// <remarks>
/// Po zmene je nutno zavolat setCallbackForComm(...)
/// </remarks>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function setPatientsChangedCallback(AHandle: TCallbackHandle;
  AOnChanged: TStdcallCallback; ACustomPayload: Pointer): Integer; stdcall;
/// <summary>
/// Registrace callbacku pro zmenu stavu Vitejte.
/// </summary>
/// <param name="AHandle">
/// Pointer na callback objekt ziskany pomoci createCallbackHandle()
/// </param>
/// <param name="AOnStateChanged">
/// Callback
/// </param>
/// <param name="ACustomPayload">
/// Parametr, ktery bude predan callbacku pri jeho vyvolani.
/// </param>
/// <remarks>
/// Po zmene je nutno zavolat setCallbackForComm(...)
/// </remarks>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function setStateChangedCallback(AHandle: TCallbackHandle;
  AOnStateChanged: TStdcallCallbackState; ACustomPayload: Pointer)
  : Integer; stdcall;
/// <summary>
/// Registrace callbacku pro chybu ve Vitejte.
/// </summary>
/// <param name="AHandle">
/// Pointer na callback objekt ziskany pomoci createCallbackHandle()
/// </param>
/// <param name="AOnError">
/// Callback
/// </param>
/// <param name="ACustomPayload">
/// Parametr, ktery bude predan callbacku pri jeho vyvolani.
/// </param>
/// <remarks>
/// Po zmene je nutno zavolat setCallbackForComm(...)
/// </remarks>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function setErrorCallback(AHandle: TCallbackHandle;
  AOnError: TStdcallCallbackError; ACustomPayload: Pointer): Integer; stdcall;
/// <summary>
/// Nastavi vybranemu pacientovi priznak odbaven.
/// </summary>
/// <param name="AHandle">
/// Pointer na komunikacni objekt ziskany pomoci createCommunicationHandle()
/// </param>
/// <param name="APatientID">
/// ID ciloveho pacienta.
/// </param>
/// <returns>
/// 0 pokud OK, jinak vizte vyse
/// </returns>
function setCallbackForComm(AHandle: TCommHandle;
  ACallbackHandle: TCallbackHandle): Integer; stdcall;
/// <summary>
/// Uvolni pole stringu alokovane pomoci VitejteLib
/// </summary>
procedure freeStringArray(var AStringArray: PStringArray;
  ASize: Integer); stdcall;
/// <summary>
/// Uvolni string alokovany pomoci VitejteLib
/// </summary>
procedure freeString(var AStr: PAnsiChar); stdcall;
/// <summary>
/// Uvolni pole pacientu alokovane pomoci VitejteLib
/// </summary>
procedure freePatients(APatients: PPacientArray; ACount: Integer); stdcall;
/// <summary>
/// Uvolni objekt pro registraci callbacku
/// </summary>
function deleteCallbackHandle(AHandle: TCommHandle): Integer; stdcall;
/// <summary>
/// Uvolni objekt pro komunikaci
/// </summary>
/// <remarks>
/// Delphi je peklo
/// </remarks>
function deleteCommunicationHandle(AHandle: TCommHandle): Integer; stdcall;

implementation

const
  DLL_NAME = 'VitejteLib.dll';

var
  DllHandle: THandle = 0;

type
  TCreateCommunicationHandle = function(): TCommHandle; stdcall;
  TCreateCallbackHandle = function(): TCallbackHandle; stdcall;
  TGetPat = function(AHandle: TCommHandle; var APatients: PPacientArray;
    var ACount: Integer): Integer; stdcall;
  TFindDevices = function(AHandle: TCommHandle; var AIDList: PStringArray;
    var Size: Integer): Integer; stdcall;
  TSetDevice = function(AHandle: TCommHandle; AID: PAnsiChar): Integer; stdcall;
  TStartPingTimer = function(AHandle: TCommHandle): Integer; stdcall;
  TStopPingTimer = function(AHandle: TCommHandle): Integer; stdcall;
  TRegisterClient = function(AHandle: TCommHandle; APort: Integer;
    AQueues: PAnsiChar): Integer; stdcall;
  TUnRegisterClient = function(AHandle: TCommHandle): Integer; stdcall;
  TGetVitejteIP = function(AHandle: TCommHandle; var AIP: PAnsiChar)
    : Integer; stdcall;
  TGetVitejteID = function(AHandle: TCommHandle; var AIP: PAnsiChar)
    : Integer; stdcall;
  TGetInfo = function(AHandle: TCommHandle; var AIP: PAnsiChar)
    : Integer; stdcall;
  TClearPatient = function(Handle: TCommHandle; APatientID: Integer)
    : Integer; stdcall;
  TSetPatientsChangedCallback = function(AHandle: TCallbackHandle;
    AOnChanged: TStdcallCallback; ACustomPayload: Pointer): Integer; stdcall;
  TSetStateChangedCallback = function(AHandle: TCallbackHandle;
    AOnStateChanged: TStdcallCallbackState; ACustomPayload: Pointer)
    : Integer; stdcall;
  TSetErrorCallback = function(AHandle: TCallbackHandle;
    AOnError: TStdcallCallbackError; ACustomPayload: Pointer): Integer; stdcall;
  TSetCallbackForComm = function(AHandle: TCommHandle;
    ACallbackHandle: TCallbackHandle): Integer; stdcall;
  TFreeStringArray = procedure(var AStringArray: PStringArray;
    ASize: Integer); stdcall;
  TFreeString = procedure(var AStr: PAnsiChar); stdcall;
  TFreePatients = procedure(APatients: PPacientArray; ACount: Integer); stdcall;
  TDeleteCallbackHandle = function(AHandle: TCommHandle): Integer; stdcall;
  TDeleteCommunicationHandle = function(AHandle: TCommHandle): Integer; stdcall;

function createCommunicationHandle: TCommHandle; stdcall;
var
  func: TCreateCommunicationHandle;
begin
  func := nil;
  WUGetProcAdressEx('createCommunicationHandle', @func, PChar(DLL_NAME),
    DllHandle);
  Result := func();
end;

function createCallbackHandle: TCallbackHandle; stdcall;
var
  func: TCreateCallbackHandle;
begin
  func := nil;
  WUGetProcAdressEx('createCallbackHandle', @func, PChar(DLL_NAME), DllHandle);
  Result := func();
end;

function getPat(AHandle: TCommHandle; var APatients: PPacientArray;
  var ACount: Integer): Integer; stdcall;
var
  func: TGetPat;
begin
  func := nil;
  WUGetProcAdressEx('getPat', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle, APatients, ACount);
end;

function findDevices(AHandle: TCommHandle; var AIDList: PStringArray;
  var Size: Integer): Integer; stdcall;
var
  func: TFindDevices;
begin
  func := nil;
  WUGetProcAdressEx('findDevices', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle, AIDList, Size);
end;

function setDevice(AHandle: TCommHandle; AID: PAnsiChar): Integer; stdcall;
var
  func: TSetDevice;
begin
  func := nil;
  WUGetProcAdressEx('setDevice', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle, AID);
end;

function startPingTimer(AHandle: TCommHandle): Integer; stdcall;
var
  func: TStartPingTimer;
begin
  func := nil;
  WUGetProcAdressEx('startPingTimer', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle);
end;

function stopPingTimer(AHandle: TCommHandle): Integer; stdcall;
var
  func: TStopPingTimer;
begin
  func := nil;
  WUGetProcAdressEx('stopPingTimer', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle);
end;

function registerClient(AHandle: TCommHandle; APort: Integer;
  AQueues: PAnsiChar): Integer; stdcall;
var
  func: TRegisterClient;
begin
  func := nil;
  WUGetProcAdressEx('registerClient', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle, APort, AQueues);
end;

function unRegisterClient(AHandle: TCommHandle): Integer; stdcall;
var
  func: TUnRegisterClient;
begin
  func := nil;
  WUGetProcAdressEx('unRegisterClient', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle);
end;

function getVitejteIP(AHandle: TCommHandle; var AIP: PAnsiChar)
  : Integer; stdcall;
var
  func: TGetVitejteIP;
begin
  func := nil;
  WUGetProcAdressEx('getVitejteIP', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle, AIP);
end;

function getVitejteID(AHandle: TCommHandle; var AIP: PAnsiChar)
  : Integer; stdcall;
var
  func: TGetVitejteID;
begin
  func := nil;
  WUGetProcAdressEx('getVitejteID', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle, AIP);
end;

function getInfo(AHandle: TCommHandle; var AIP: PAnsiChar): Integer; stdcall;
var
  func: TGetInfo;
begin
  func := nil;
  WUGetProcAdressEx('getInfo', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle, AIP);
end;

function clearPatient(Handle: TCommHandle; APatientID: Integer)
  : Integer; stdcall;
var
  func: TClearPatient;
begin
  func := nil;
  WUGetProcAdressEx('clearPatient', @func, PChar(DLL_NAME), DllHandle);
  Result := func(Handle, APatientID);
end;

function setPatientsChangedCallback(AHandle: TCallbackHandle;
  AOnChanged: TStdcallCallback; ACustomPayload: Pointer): Integer; stdcall;
var
  func: TSetPatientsChangedCallback;
begin
  func := nil;
  WUGetProcAdressEx('setPatientsChangedCallback', @func, PChar(DLL_NAME),
    DllHandle);
  Result := func(AHandle, AOnChanged, ACustomPayload);
end;

function setStateChangedCallback(AHandle: TCallbackHandle;
  AOnStateChanged: TStdcallCallbackState; ACustomPayload: Pointer)
  : Integer; stdcall;
var
  func: TSetStateChangedCallback;
begin
  func := nil;
  WUGetProcAdressEx('setStateChangedCallback', @func, PChar(DLL_NAME),
    DllHandle);
  Result := func(AHandle, AOnStateChanged, ACustomPayload);
end;

function setErrorCallback(AHandle: TCallbackHandle;
  AOnError: TStdcallCallbackError; ACustomPayload: Pointer): Integer; stdcall;
var
  func: TSetErrorCallback;
begin
  func := nil;
  WUGetProcAdressEx('setErrorCallback', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle, AOnError, ACustomPayload);
end;

function setCallbackForComm(AHandle: TCommHandle;
  ACallbackHandle: TCallbackHandle): Integer; stdcall;
var
  func: TSetCallbackForComm;
begin
  func := nil;
  WUGetProcAdressEx('setCallbackForComm', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle, ACallbackHandle);
end;

procedure freeStringArray(var AStringArray: PStringArray;
  ASize: Integer); stdcall;
var
  func: TFreeStringArray;
begin
  func := nil;
  WUGetProcAdressEx('freeStringArray', @func, PChar(DLL_NAME), DllHandle);
  func(AStringArray, ASize);
end;

procedure freeString(var AStr: PAnsiChar); stdcall;
var
  func: TFreeString;
begin
  func := nil;
  WUGetProcAdressEx('freeString', @func, PChar(DLL_NAME), DllHandle);
  func(AStr);
end;

procedure freePatients(APatients: PPacientArray; ACount: Integer); stdcall;
var
  func: TFreePatients;
begin
  func := nil;
  WUGetProcAdressEx('freePatients', @func, PChar(DLL_NAME), DllHandle);
  func(APatients, ACount);
end;

function deleteCallbackHandle(AHandle: TCommHandle): Integer; stdcall;
var
  func: TDeleteCallbackHandle;
begin
  func := nil;
  WUGetProcAdressEx('deleteCallbackHandle', @func, PChar(DLL_NAME), DllHandle);
  Result := func(AHandle);
end;

function deleteCommunicationHandle(AHandle: TCommHandle): Integer; stdcall;
var
  func: TDeleteCommunicationHandle;
begin
  func := nil;
  WUGetProcAdressEx('deleteCommunicationHandle', @func, PChar(DLL_NAME),
    DllHandle);
  Result := func(AHandle);
end;

end.
